/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2017, Red Hat, Inc., and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.wildfly.test.integration.microprofile.config.smallrye.management.config_source.from_class;

import jakarta.inject.Inject;
import jakarta.ws.rs.ApplicationPath;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Application;
import jakarta.ws.rs.core.Response;

import org.eclipse.microprofile.config.inject.ConfigProperty;

/**
 * @author <a href="http://jmesnil.net/">Jeff Mesnil</a> (c) 2017 Red Hat inc.
 */
@ApplicationPath("/custom-config-source")
public class TestApplication extends Application {

    @Path("/test")
    public static class Resource {

        @Inject
        @ConfigProperty(name = CustomConfigSource.PROP_NAME)
        String prop;

        @Inject
        @ConfigProperty(name = CustomConfigSourceServiceLoader.PROP_NAME)
        String propFromServiceLoader;

        @Inject
        @ConfigProperty(name = CustomConfigSource.PROP_NAME_OVERRIDEN_BY_SERVICE_LOADER)
        String propOverridenByServiceLoader;

        @Inject
        @ConfigProperty(name = CustomConfigSourceAServiceLoader.PROP_NAME_SAME_ORDINALITY_OVERRIDE)
        String propSameOrdinalityOverridenByFqcn;

        @GET
        @Produces("text/plain")
        public Response doGet() {
            StringBuilder text = new StringBuilder();
            text.append(CustomConfigSource.PROP_NAME + " = " + prop + "\n");
            text.append(CustomConfigSourceServiceLoader.PROP_NAME + " = " + propFromServiceLoader + "\n");
            text.append(CustomConfigSource.PROP_NAME_OVERRIDEN_BY_SERVICE_LOADER + " = " + propOverridenByServiceLoader + "\n");
            text.append(CustomConfigSourceAServiceLoader.PROP_NAME_SAME_ORDINALITY_OVERRIDE + " = " +
                    propSameOrdinalityOverridenByFqcn + "\n");
            return Response.ok(text).build();
        }
    }
}
